// src/controllers/authController.ts
import { Request, Response } from 'express';
import { comparePassword, generateToken } from '../services/authService';
import { User } from '../entities/User';

export const login = async (req: Request, res: Response) => {
    try {
        const { rg, password } = req.body;
        const user = await User.findOne({ where: { rg } });

        console.log('Rg', rg);
        console.log('Usuario', user);

        if (!user || !(await comparePassword(password, user.password))) {
            return res.status(401).json({ message: 'Credenciais inválidas' });
        }

        const accessToken = generateToken({ userId: user.id, rg: user.rg });

        res.json({ accessToken });
    } catch (error) {
        console.error(error);
        res.status(500).json({ message: 'Erro no processo de login' });
    }
};

export const createUser = async (req: Request, res: Response) => {
    try {
        const { username, password, fullName, email, hasApiAccess } = req.body;

        const newUser = new User();
        newUser.username = username;
        newUser.fullName = fullName;
        newUser.email = email;
        newUser.hasApiAccess = hasApiAccess;

        if (password) {
            await newUser.setPassword(password);
        }

        // Salva o novo usuário no banco de dados
        await newUser.save();

        res.json({ message: 'Usuário criado com sucesso' });
    } catch (error) {

        res.status(500).json({ message: 'Erro ao criar o usuário' });
    }
};
