import { Request, Response } from 'express';
import { In, getRepository } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { InventariosMateriais } from '../entities/InventariosMateriais';
import { Inventarios } from '../entities/Inventarios';
import { User } from '../entities/User';

class InventarioController {
    static async listAll(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        const { tipoEquipamento } = req.params;
        try {
            const repositoryUsuario = getRepository(User);
            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }
            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const repository = getRepository(Inventarios);
            const inventarios = await repository.find({
                where: {
                    viatura: {
                        unidade: {
                            id: In(unidadesIds)
                        }
                    }
                },
                relations: ['viatura', 'viatura.imagens', 'tipoEquipamento'],
                order: { id: 'ASC' }
            });

            return res.json(inventarios);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async list(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        const { tipoEquipamento } = req.params;
        try {
            const repositoryUsuario = getRepository(User);
            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }
            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const repository = getRepository(Inventarios);
            const inventarios = await repository.find({
                where: {
                    tipoEquipamento,
                    viatura: {
                        unidade: {
                            id: In(unidadesIds)
                        }
                    }
                },
                relations: ['viatura', 'viatura.imagens'],
                order: { id: 'ASC' }
            });

            return res.json(inventarios);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Inventarios);
            const inventario = await repository.findOne({ where: { id }, relations: ['inventarioMateriais', 'inventarioMateriais.equipamento', 'inventarioMateriais.unidadeMaterial', 'inventarioMateriais.categoria', 'viatura', 'viatura.imagens', 'viatura.unidade'] });
            if (!inventario) {
                return res.status(404).json({ message: "Inventário não encontrado" });
            }
            return res.json(inventario);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, viatura, equipamentos, situacao, tipoEquipamento } = req.body;

            console.log("Body create inventário", req.body);
            console.log('Files', req.files);

            const repositoryMaterial = getRepository(InventariosMateriais);
            const repositoryInventario = getRepository(Inventarios);

            let existeRegistro = await repositoryInventario.findOne({ where: { descricao, tipoEquipamento, viatura } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um inventário com esta descrição.");
            }
            existeRegistro = await repositoryInventario.findOne({ where: { viatura, tipoEquipamento } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um inventário para esta viatura.");
            }

            const inventario = new Inventarios();
            inventario.descricao = descricao;
            inventario.viatura = viatura;
            inventario.tipoEquipamento = tipoEquipamento;
            inventario.situacao = situacao;

            const result = await repositoryInventario.save(inventario);

            if (tipoEquipamento == 1) {
                for (const equipamento of equipamentos) {
                    const inventarioMaterial = new InventariosMateriais();
                    inventarioMaterial.categoria = equipamento.categoria;
                    inventarioMaterial.quantidade = equipamento.quantidade;
                    inventarioMaterial.equipamento = equipamento.id;
                    inventarioMaterial.unidadeMaterial = equipamento.unidadeMaterial;
                    inventarioMaterial.inventario = result;

                    await repositoryMaterial.save(inventarioMaterial);
                }
            } else {
                for (const equipamento of equipamentos) {
                    const inventarioMaterial = new InventariosMateriais();
                    inventarioMaterial.equipamento = equipamento.id;
                    inventarioMaterial.unidadeMaterial = equipamento.unidadeMaterial;
                    inventarioMaterial.inventario = result;

                    await repositoryMaterial.save(inventarioMaterial);
                }
            }

            return res.status(200).json("Inventario criado.");
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o inventário. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, viatura, equipamentos, situacao, tipoEquipamento, equipamentosRemover } = req.body;
            console.log('Body', req.body);

            const repository = getRepository(Inventarios);
            const repositoryMaterial = getRepository(InventariosMateriais);

            let existeRegistro = await repository.findOne({ where: { descricao, tipoEquipamento } });
            console.log('Registro encontrando', existeRegistro);
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa descrição.");
            }

            existeRegistro = await repository.findOne({ where: { viatura, tipoEquipamento } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa viatura.");
            }

            const inventarioToUpdate = await repository.findOne(id);
            if (!inventarioToUpdate) {
                return res.status(404).json("Inventário não encontrado");
            }



            console.log('Inventario encontrado', inventarioToUpdate);
            console.log('Equipamentos', equipamentos);
            console.log('Equipamentos Remover', equipamentosRemover);

            const data = { descricao, viatura, situacao };

            repository.merge(inventarioToUpdate, data);
            const result = await repository.save(inventarioToUpdate);

            if (tipoEquipamento == 1) {
                for (const e of equipamentos) {
                    if (!e.id) {
                        const inventarioMaterial = new InventariosMateriais();
                        inventarioMaterial.categoria = e.categoria;
                        inventarioMaterial.quantidade = e.quantidade;
                        inventarioMaterial.equipamento = e.equipamento.id;
                        inventarioMaterial.unidadeMaterial = e.unidadeMaterial;
                        inventarioMaterial.inventario = inventarioToUpdate;

                        await repositoryMaterial.save(inventarioMaterial);
                    }
                }
            } else {
                for (const e of equipamentos) {
                    if (!e.id) {
                        const inventarioMaterial = new InventariosMateriais();
                        inventarioMaterial.equipamento = e.equipamento.id;
                        inventarioMaterial.unidadeMaterial = e.unidadeMaterial;
                        inventarioMaterial.inventario = inventarioToUpdate;

                        await repositoryMaterial.save(inventarioMaterial);
                    }
                }
            }

            for (const r of equipamentosRemover) {
                await repositoryMaterial.softDelete(r.id);
            }

            console.log('Resultado', result);

            return res.json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json("Erro ao salvar a inventário. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Inventarios);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Inventário não encontrada');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a inventário. Tente novamente.");
        }
    }

    static async createInventario(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, viatura, equipamentos, situacao, tipoEquipamento } = req.body;

            console.log("Body create inventário", req.body);

            const repositoryInventario = getRepository(Inventarios);

            let existeRegistro = await repositoryInventario.findOne({ where: { descricao, tipoEquipamento: tipoEquipamento.id, viatura: viatura.id } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um inventário com esta descrição.");
            }
            existeRegistro = await repositoryInventario.findOne({ where: { viatura: viatura.id, tipoEquipamento: tipoEquipamento.id } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um inventário para esta viatura.");
            }

            const inventario = new Inventarios();
            inventario.descricao = descricao;
            inventario.viatura = viatura.id;
            inventario.tipoEquipamento = tipoEquipamento.id;
            inventario.situacao = situacao;

            const result = await repositoryInventario.save(inventario);
            const invF = await repositoryInventario.findOne({ where: { id: result.id }, relations: ['viatura', 'tipoEquipamento'] })

            return res.status(200).json(invF);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o inventário. Tente novamente.');
        }
    }

    static async updateInventario(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, viatura, equipamentos, situacao, tipoEquipamento, equipamentosRemover } = req.body;
            console.log('Body', req.body);

            const repository = getRepository(Inventarios);


            let existeRegistro = await repository.findOne({ where: { descricao, tipoEquipamento: tipoEquipamento.id } });
            console.log('Registro encontrando', existeRegistro);
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa descrição.");
            }

            existeRegistro = await repository.findOne({ where: { viatura: viatura.id, tipoEquipamento: tipoEquipamento.id } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa viatura.");
            }

            const inventarioToUpdate = await repository.findOne(id);
            if (!inventarioToUpdate) {
                return res.status(404).json("Inventário não encontrado");
            }



            console.log('Inventario encontrado', inventarioToUpdate);

            const data = { descricao, viatura: viatura.id, situacao, tipoEquipamento: tipoEquipamento.id };

            repository.merge(inventarioToUpdate, data);
            const result = await repository.save(inventarioToUpdate);



            //console.log('Resultado', result);

            //return res.json(result);
            return res.json('Atualizado');
        } catch (error) {
            console.error(error);
            return res.status(500).json("Erro ao salvar a inventário. Tente novamente.");
        }
    }
}

export default InventarioController;
