import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { ItensImplemento } from '../entities/ItensImplemento';

class ItemImplementoController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(ItensImplemento);
            const itens = await repository.find({ order: { descricao: 'ASC' } })
            return res.json(itens);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(ItensImplemento);
            const item = await repository.findOne(id);
            if (!item) {
                return res.status(404).json({ message: "Item do implemento não encontrado" });
            }
            return res.json(item);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, comentarios, situacao } = req.body;
            let imagens = [];

            console.log("Body create item do implemento", req.body);
            console.log('Files', req.files);

            const repository = getRepository(ItensImplemento);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um item com esta descrição.");
            }

            const item = repository.create(req.body);
            const result: any = await repository.save(item);

            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o item do implemento. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, comentarios, situacao } = req.body;
            console.log('Body', req.body);

            const repository = getRepository(ItensImplemento);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um item com essa descrição.");
            }

            const itemToUpdate = await repository.findOne(id);
            if (!itemToUpdate) {
                return res.status(404).json("Item não encontrado");
            }

            repository.merge(itemToUpdate, req.body.data);
            const result = await repository.save(itemToUpdate);

            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar o item do implemento. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(ItensImplemento);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Item não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a item do implemento. Tente novamente.");
        }
    }
}

export default ItemImplementoController;
