import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { ItensVeiculo } from '../entities/ItensVeiculo';
import fs from 'fs';
import path from 'path';
import { Imagens } from '../entities/Imagens';

class ItemVeiculoController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(ItensVeiculo);
            const itens = await repository.find({ order: { descricao: 'ASC' } })
            return res.json(itens);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(ItensVeiculo);
            const item = await repository.findOne(id);
            if (!item) {
                return res.status(404).json({ message: "Item de veículo não encontrado" });
            }
            return res.json(item);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, comentarios, situacao } = req.body;
            let imagens = [];

            console.log("Body create item de veiculo", req.body);
            console.log('Files', req.files);

            const repository = getRepository(ItensVeiculo);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um item com esta descrição.");
            }

            const item = repository.create(req.body);
            const result: any = await repository.save(item);

            if (req.files && Array.isArray(req.files)) {
                const directory = path.join(__dirname, '..', '..', 'uploads', 'itensveiculo', `${result.id}`);
                if (!fs.existsSync(directory)) {
                    fs.mkdirSync(directory, { recursive: true });
                }

                for (const file of req.files) {
                    const fileExtension = file.originalname.split('.').pop(); // Obter a extensão do arquivo
                    const fileName = `${new Date().getTime().toString()}.${fileExtension}`; // Nome do arquivo com a extensão
                    const filePath = path.join(directory, fileName);

                    await fs.writeFileSync(filePath, file.buffer);

                    // Cria a entrada de imagem no banco de dados
                    const imagemRepository = getRepository(Imagens);
                    const imagem = imagemRepository.create({
                        path: path.relative(path.join(__dirname, '..', '..'), filePath),
                        viatura_id: result.id
                    });
                    await imagemRepository.save(imagem);
                    imagens.push(imagem);
                }
            }

            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o item de veículo. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, comentarios, situacao } = req.body;
            console.log('Body', req.body);

            const repository = getRepository(ItensVeiculo);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um item com essa descrição.");
            }

            const itemToUpdate = await repository.findOne(id);
            if (!itemToUpdate) {
                return res.status(404).json("Item não encontrado");
            }

            repository.merge(itemToUpdate, req.body.data);
            const result = await repository.save(itemToUpdate);

            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar o item do veículo. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(ItensVeiculo);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Item não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a item de veículo. Tente novamente.");
        }
    }
}

export default ItemVeiculoController;
