// src/controllers/RolesController.ts

import { Request, Response } from 'express';
import { LessThanOrEqual, MoreThanOrEqual, getRepository } from 'typeorm';
import { Role } from '../entities/Role';
import { Permission } from '../entities/Permission';


export const getPermissions = async (req: Request, res: Response) => {
    try {
        const repository = getRepository(Permission);
        const permissions = await repository.find({ where: { deletedAt: null }, order: { modulo: 'ASC', createdAt: 'ASC' } });

        res.json(permissions);
    } catch (error) {
        console.error(error);
        res.status(500).send('Erro ao buscar permissões.');
    }
};

export const showPermission = async (req: Request, res: Response) => {
    const { id } = req.params;
    try {

        const repository = getRepository(Permission);
        const permission = await repository.findOne(id);

        res.json(permission);
    } catch (error) {
        console.error(error);
        res.status(500).send(`Erro ao buscar permissão ${id}.`);
    }
};

export const createPermission = async (req: Request, res: Response) => {
    const { name, description } = req.body;

    const repository = getRepository(Permission);

    try {
        const permission = new Permission();
        permission.name = name;
        permission.description = description;


        const novaPermission = repository.create(permission);

        const permissionSalva = await repository.save(novaPermission);


        res.send({ role: permissionSalva });
    } catch (error) {
        console.error(error)
        res.status(500).send({ codigo: 500, mensagem: `Erro ao tentar criar a permission` });
    }
};

export const updateInscricao = (req: Request, res: Response) => {
    res.send('Atualizar inscrição');
};

export const deletePermmission = async (req: Request, res: Response) => {
    const { id } = req.params;
    try {


        const repository = getRepository(Permission);
        const permission = await repository.findOne(id);
        if (permission) {
            repository.softRemove(permission);
            return res.send('Permissão excluída');
        } else {
            return res.status(400).send('Permissão não encontrada');
        }
    } catch (error) {
        console.error(error);
        return res.status(500).send(`Erro ao deletar permissão ${id}.`);
    }
};
