// src/controllers/RolesController.ts

import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Role } from '../entities/Role';
import { PermissionRepository } from '../repositories';


export const getRoles = async (req: Request, res: Response) => {
    try {
        const rolesRepository = getRepository(Role);
        const roles = await rolesRepository.find({ where: { deletedAt: null }, relations: ["permissions"] });

        res.json(roles);
    } catch (error) {
        console.error(error);
        res.status(500).send('Erro ao buscar roles.');
    }
};

export const showRole = async (req: Request, res: Response) => {
    const { id } = req.params;
    try {

        const rolesRepository = getRepository(Role);
        const role = await rolesRepository.findOne(id, { relations: ["permissions"] });

        res.json(role);
    } catch (error) {
        console.error(error);
        res.status(500).send(`Erro ao buscar role ${id}.`);
    }
};

export const createRoles = async (req: Request, res: Response) => {
    const { descricao, comentarios, situacao, permissions } = req.body;

    console.log('Body', req.body);

    const rolesRepository = getRepository(Role);
    try {
        const role = new Role();
        role.name = descricao;
        role.description = descricao;
        role.coments = comentarios;
        role.situacao = situacao;

        if (permissions && permissions.length > 0) {
            const permissionsExists = await PermissionRepository().findByIds(
                permissions
            );
            role.permissions = permissionsExists;
        }

        const novaRole = rolesRepository.create(role);
        const roleSalva = await rolesRepository.save(novaRole);

        return res.status(200).json(roleSalva);
    } catch (error) {
        console.error(error);
        return res.status(500).json('Erro ao tentar criar o Perfil');
    }
};

export const updateRole = async (req: Request, res: Response) => {
    const { descricao, comentarios, situacao, permissions } = req.body;
    const { id } = req.params;

    console.log('Body', req.body);


    try {
        const rolesRepository = getRepository(Role);

        const existeRegistro = await rolesRepository.findOne({ where: { description: descricao } });
        if (existeRegistro && existeRegistro.id !== id) {
            return res.status(400).json("Já existe um perfil com essa descrição.");
        }

        const roleToUpdate = await rolesRepository.findOne(id);
        if (!roleToUpdate) {
            return res.status(404).json("Perfil não encontrado");
        }

        roleToUpdate.name = descricao;
        roleToUpdate.description = descricao;
        roleToUpdate.coments = comentarios;
        roleToUpdate.situacao = situacao;

        if (permissions && permissions.length > 0) {
            const permissionsExists = await PermissionRepository().findByIds(
                permissions
            );
            roleToUpdate.permissions = permissionsExists;
        }

        console.log('Role', roleToUpdate)

        const roleSalva = await rolesRepository.save(roleToUpdate);

        return res.status(200).json(roleSalva);
    } catch (error) {
        console.error(error);
        return res.status(500).json('Erro ao tentar salvar o Perfil');
    }
};

export const deleteRole = async (req: Request, res: Response) => {
    const { id } = req.params;
    try {

        console.log('ID deletar', id)

        const rolesRepository = getRepository(Role);
        const role = await rolesRepository.findOne(id);
        if (role) {
            console.log('Encontrou perfil', role)
            rolesRepository.softRemove(role);
            return res.send('Perfil excluído');
        } else {
            return res.status(400).send('Perfil não encontrado');
        }
    } catch (error) {
        console.error(error);
        return res.status(500).send(`Erro ao deletar perfil ${id}.`);
    }
};


