import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { Status } from '../entities/Status';

class StatusController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(Status);
            const listaStatus = await repository.find({ order: { descricao: 'ASC' } })
            return res.json(listaStatus);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Status);
            const status = await repository.findOne(id);
            if (!status) {
                return res.status(400).json({ message: "Status não encontrado" });
            }
            return res.json(status);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, situacao } = req.body;

            const repository = getRepository(Status);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um status com esta descrição.");
            }

            const status = repository.create(req.body);
            const result: any = await repository.save(status);

            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o status. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, situacao } = req.body;


            const repository = getRepository(Status);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um status com essa descrição.");
            }

            const statusToUpdate = await repository.findOne(id);
            if (!statusToUpdate) {
                return res.status(404).json("Status não encontrado.");
            }

            repository.merge(statusToUpdate, req.body.data);
            const result = await repository.save(statusToUpdate);

            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar o status. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Status);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(400).json('Status não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir o status. Tente novamente.");
        }
    }
}

export default StatusController;
