import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Subarea } from '../entities/Subarea';

class SubareaController {
    static async getSubareas(req: Request, res: Response): Promise<Response> {
        try {
            const subareaRepository = getRepository(Subarea);
            const subareas = await subareaRepository.find({ relations: ["igrejas", "area"] });
            return res.json(subareas);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async getSubarea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const subareaRepository = getRepository(Subarea);
            const subarea = await subareaRepository.findOne(id, { relations: ["igrejas", "area"] });
            if (!subarea) {
                return res.status(404).json({ message: "Subarea not found" });
            }
            return res.json(subarea);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async createSubarea(req: Request, res: Response): Promise<Response> {
        try {
            const subareaRepository = getRepository(Subarea);
            const subarea = subareaRepository.create(req.body);
            const result = await subareaRepository.save(subarea);
            return res.status(201).json(result);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async updateSubarea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const subareaRepository = getRepository(Subarea);
            const updatedSubarea = await subareaRepository.findOne(id);
            if (!updatedSubarea) {
                return res.status(404).json({ message: "Subarea not found" });
            }

            delete req.body.userId;
            await subareaRepository.update(id, req.body);

            return res.json(updatedSubarea);
        } catch (error) {
            console.error("Erro update subarea", error);
            return res.status(500).json({ error: error.message });
        }
    }

    static async deleteSubarea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const subareaRepository = getRepository(Subarea);
            const result = await subareaRepository.delete(id);
            if (result.affected === 0) {
                return res.status(404).json({ message: "Subarea not found" });
            }
            return res.status(204).send();
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default SubareaController;