import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { Status } from '../entities/Status';
import { ViaturaStatus } from '../entities/ViaturaStatus';
import { User } from '../entities/User';

class ViaturaStatusController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(ViaturaStatus);
            const listaStatus = await repository.find({ order: { data: 'ASC' } })
            return res.json(listaStatus);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(ViaturaStatus);
            const status = await repository.findOne(id);
            if (!status) {
                return res.status(400).json({ message: "Status não encontrado" });
            }
            return res.json(status);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, situacao, viatura, unidade, data, status, userId } = req.body;

            console.log('Body', req.body);

            const repository = getRepository(ViaturaStatus);
            const viaturaStatus = new ViaturaStatus();
            viaturaStatus.data = data;
            viaturaStatus.status = status;
            viaturaStatus.unidade = unidade;
            viaturaStatus.viatura = viatura;
            viaturaStatus.usuario = userId;

            const result: any = await repository.save(viaturaStatus);
            const objeto = await repository.findOne({ where: { id: result.id }, relations: ['status'] })

            return res.status(200).json(objeto);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o status. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, situacao } = req.body;


            const repository = getRepository(ViaturaStatus);

            const statusToUpdate = await repository.findOne(id);
            if (!statusToUpdate) {
                return res.status(404).json("Status não encontrado.");
            }

            repository.merge(statusToUpdate, req.body.data);
            const result = await repository.save(statusToUpdate);

            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar o status. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Status);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(400).json('Status não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir o status. Tente novamente.");
        }
    }

    static async getUltimo(req: Request, res: Response): Promise<Response> {
        try {
            const { viatura_id } = req.params;
            const repository = getRepository(ViaturaStatus);
            const status = await repository.findOne({ where: { viatura: viatura_id }, order: { id: 'DESC' }, relations: ['status'] });
            if (!status) {
                return res.status(400).json({ message: "Status não encontrado" });
            }
            return res.json(status);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default ViaturaStatusController;
