import { Column, CreateDateColumn, Entity, JoinColumn, JoinTable, ManyToMany, ManyToOne, OneToMany, OneToOne, PrimaryColumn, PrimaryGeneratedColumn } from "typeorm";
import { MaterialBase } from "./MaterialBase";
import { BaseEntity } from "./BaseEntity";
import { InventariosMateriais } from "./InventariosMateriais";
import { TiposEquipamento } from "./TiposEquipamento";
import { Imagens } from "./Imagens";
import { UnidadesMateriais } from "./UnidadesMateriais";

@Entity("equipamentos")
export class Equipamentos extends BaseEntity {
    @PrimaryGeneratedColumn({ name: "id", type: "int" })
    id: number;

    @Column({ name: "descricao", nullable: true })
    descricao: string

    @Column({ name: "comentarios", nullable: true })
    comentarios: string

    @Column({ name: "modelo", nullable: true })
    modelo: string

    @Column({ name: "patrimonio", nullable: true })
    patrimonio: string

    @Column({ name: "codigo", nullable: true })
    codigo: string

    @Column({ name: "urlVideo", nullable: true })
    urlVideo: string

    @Column({ name: "urlImagem", nullable: true })
    urlImagem: string;

    @OneToMany(() => Imagens, imagem => imagem.equipamento)
    imagens: Imagens[];

    @ManyToOne(() => TiposEquipamento, tipo => tipo.equipamentos)
    @JoinColumn({ name: 'tipo_equipamento_id' })
    tipoEquipamento: TiposEquipamento;

    @OneToMany(() => InventariosMateriais, inventarioMateriais => inventarioMateriais.equipamento)
    inventariosMateriais: InventariosMateriais[];

    @OneToMany(() => UnidadesMateriais, unidadesMateriais => unidadesMateriais.equipamento)
    unidadesMateriais: UnidadesMateriais[];

    @Column({ type: "varchar", length: 15 })
    situacao: string;
}
