import { Column, CreateDateColumn, Entity, JoinColumn, JoinTable, ManyToMany, ManyToOne, OneToMany, PrimaryColumn, PrimaryGeneratedColumn } from "typeorm";
import { BaseEntity } from "./BaseEntity";
import { Area } from "./Area";
import { Subarea } from "./Subarea";
import { Viaturas } from "./Viaturas";
import { UnidadesMateriais } from "./UnidadesMateriais";

@Entity("unidades")
export class Unidades extends BaseEntity {
    @PrimaryGeneratedColumn({ name: "id", type: "int" })
    id: number;

    @Column({ name: "codigo", nullable: true })
    codigo: string

    @Column({ name: "nome", nullable: true })
    nome: string

    @Column({ name: "numero", nullable: true })
    numero: string

    @Column({ name: "endereco", nullable: true })
    logradouro: string

    @Column({ name: "bairro", nullable: true })
    bairro: string

    @Column({ name: "cidade", nullable: true })
    cidade: string

    @Column({ name: "complemento", nullable: true })
    complemento: string

    @Column({ name: "cep", nullable: true })
    cep: string

    @Column({ name: "zona", nullable: true })
    zona: string

    @Column({ type: "varchar", length: 15 })
    status: string;

    @OneToMany(() => Viaturas, viatura => viatura.unidade)
    viaturas: Viaturas[];

    @ManyToOne(() => Area, area => area.unidade)
    @JoinColumn({ name: "area_id" })
    area: Area;

    @ManyToOne(() => Subarea, subarea => subarea.unidades, { nullable: true })
    @JoinColumn({ name: "subarea_id" })
    subarea: Subarea;
    @Column({ type: "int", nullable: true })
    ordem?: number;

    @OneToMany(() => UnidadesMateriais, unidadesMateriais => unidadesMateriais.unidade)
    materiais: UnidadesMateriais[];
}