// src/entities/User.ts
import { Entity, PrimaryGeneratedColumn, Column, ManyToMany, JoinTable, DeleteDateColumn, OneToMany } from 'typeorm';
import { Role } from './Role';
import { BaseEntity } from './BaseEntity';
import { hashPassword } from '../services/authService';
import { Permission } from './Permission';
import { Unidades } from './Unidades';

@Entity('users')
export class User extends BaseEntity {
    @PrimaryGeneratedColumn("uuid")
    id: string;

    @Column({ type: "varchar", length: 100, nullable: true })
    username: string;

    @Column()
    password: string;

    @Column({ type: "varchar", length: 100 })
    fullName: string;

    @Column({ type: "varchar", length: 100, unique: true })
    email: string;

    @Column({ type: "varchar", length: 100, unique: true })
    rg: string;

    @Column({ type: "varchar", length: 100 })
    celular: string;

    @Column({ type: "varchar", length: 15 })
    status: string;

    @Column({ default: false })
    hasApiAccess: boolean;

    @ManyToMany(() => Role)
    @JoinTable({
        name: "users_roles",
        joinColumns: [{ name: "user_id" }],
        inverseJoinColumns: [{ name: "role_id" }],
    })
    roles: Role[];

    @ManyToMany(() => Permission)
    @JoinTable({
        name: "users_permissions",
        joinColumns: [{ name: "user_id" }],
        inverseJoinColumns: [{ name: "permission_id" }],
    })
    permissions: Permission[];

    @ManyToMany(() => Unidades)
    @JoinTable({
        name: "users_unidades",
        joinColumns: [{ name: "user_id" }],
        inverseJoinColumns: [{ name: "unidade_id" }],
    })
    unidades: Unidades[];

    @Column({ name: "primeiro_acesso", default: false })
    primeiroAcesso: boolean;


    async setPassword(password: string): Promise<void> {
        this.password = await hashPassword(password);
    }
}