require('dotenv').config();
import "reflect-metadata";
import fs from 'fs';
import https from 'https';

import { createConnection } from "typeorm";
import express from "express";
import authRoutes from "./routes/authRoutes";
import permissionRoutes from "./routes/permissionRoutes";
import rolesRoutes from "./routes/rolesRoutes";
import userRoutes from "./routes/userRoutes";
import unidadesRoutes from "./routes/unidadesRoutes";
import viaturasRoutes from "./routes/viaturasRoutes";
import equipamentosRoutes from "./routes/equipamentosRoutes";
import inventariosRoutes from "./routes/inventariosRoutes";
import checklistsRoutes from "./routes/checkListsRoutes";
import categoriasRoutes from "./routes/categoriasRoutes";
import itensVeiculoRoutes from "./routes/itensVeiculoRoutes";
import itensImplementoRoutes from "./routes/itensImplementoRoutes";
import inventarioMaterialRoutes from "./routes/inventarioMaterialRoutes";
import areaRoutes from "./routes/areaRoutes";
import subareaRoutes from "./routes/subareaRoutes";
import statusRoutes from "./routes/statusRoutes";
import viaturaStatusRoutes from "./routes/viaturaStatusRoutes";
import tiposViaturaRoutes from "./routes/tiposViaturaRoutes";
import relatoriosRoutes from "./routes/relatorioRoutes";

const app = express();
const path = require('path');
const port = process.env.PORT || process.env.PORT_API;
const prefix = '/api';
const cors = require('cors');
const cron = require('node-cron');

const allowedOrigins = process.env.URL_PERMISSAO_CONSUMO.split(',');

createConnection()
    .then(() => {
        app.use(cors({
            origin: '*',
            methods: "GET,HEAD,PUT,PATCH,POST,DELETE",
            credentials: true,
        }));


        app.get('/', (req, res) => {
            res.send('Hello, world!');
        });
        app.use(express.json());

        // Rotas
        app.use(`${prefix}/unidades`, unidadesRoutes);
        app.use(`${prefix}/viaturas`, viaturasRoutes);
        app.use(`${prefix}/equipamentos`, equipamentosRoutes);
        app.use(`${prefix}/categorias`, categoriasRoutes);
        app.use(`${prefix}/inventarios`, inventariosRoutes);
        app.use(`${prefix}/inventariosmaterias`, inventarioMaterialRoutes);
        app.use(`${prefix}/itensveiculo`, itensVeiculoRoutes);
        app.use(`${prefix}/itensimplemento`, itensImplementoRoutes);
        app.use(`${prefix}/checklists`, checklistsRoutes);
        app.use(`${prefix}/areas`, areaRoutes);
        app.use(`${prefix}/subareas`, subareaRoutes);
        app.use(`${prefix}/auth`, authRoutes);
        app.use(`${prefix}/permissions`, permissionRoutes);
        app.use(`${prefix}/roles`, rolesRoutes);
        app.use(`${prefix}/users`, userRoutes);
        app.use(`${prefix}/status`, statusRoutes);
        app.use(`${prefix}/viaturastatus`, viaturaStatusRoutes);
        app.use(`${prefix}/tiposviatura`, tiposViaturaRoutes);
        app.use(`${prefix}/relatorios`, relatoriosRoutes);
        app.use(`/uploads`, express.static(path.join(__dirname, '..', 'uploads')));

        app.listen(port, () => {
            console.log(`Server running on https://localhost:${port}`);
            const uploadsPath = path.join(__dirname, '..', 'uploads');
            console.log(`Serving uploads from: ${uploadsPath}`);
        });
    })
    .catch((error) => console.log("TypeORM connection error: ", error));
