require = require('esm')(module /*, options*/);
module.exports = require('./index.js');

require('dotenv').config();
const express = require("express");
const cors = require('cors');
const { createConnection } = require("typeorm");
const camposRoutes = require("./routes/camposRoutes.ts");
const estadosRoutes = require("./routes/estadosRoutes.ts");
const inscricoesRoutes = require("./routes/inscricoesRoutes.ts");
const authRoutes = require("./routes/authRoutes.ts");
const precosRoutes = require("./routes/precosRoutes.ts");

const app = express();
const port = process.env.PORT || process.env.PORT_API;
const prefix = '/api';

createConnection()
    .then(() => {
        app.use(cors({
            origin: [process.env.URL_PERMISSAO_CONSUMO], // Adicione a origem do seu aplicativo React
            methods: "GET,HEAD,PUT,PATCH,POST,DELETE",
            credentials: true,
        }));

        app.listen(port, () => {
            console.log(`Server running on http://localhost:${port}`);
        });

        app.get('/', (req, res) => {
            res.send('Hello, world!');
        });

        app.use(express.json());

        // Rotas
        app.use(`${prefix}/auth`, authRoutes);
        app.use(`${prefix}/campos`, camposRoutes);
        app.use(`${prefix}/estados`, estadosRoutes);
        app.use(`${prefix}/inscricoes`, inscricoesRoutes);
        app.use(`${prefix}/precos`, precosRoutes);
    })
    .catch((error) => console.log("TypeORM connection error: ", error));
