// src/middleware/authMiddleware.ts
import { Request, Response, NextFunction } from 'express';
import { verifyToken } from '../services/authService';
import { getRepository } from 'typeorm';
import { User } from '../entities/User';
import { UUID } from 'crypto';

interface AuthenticatedRequest extends Request {
    user?: {
        id: string;
        roles: string[]; // Adicione as funções do usuário
    };
}

export const authenticateToken = async (req: AuthenticatedRequest, res: Response, next: NextFunction) => {
    const token = req.header('Authorization');

    if (!token) {
        return res.status(401).json({ message: 'Token não fornecido' });
    }

    try {
        const decoded = verifyToken(token);
        const userRepository = getRepository(User);
        const user = await userRepository.findOne(decoded.userId, { relations: ['roles'] });

        if (!user) {
            return res.status(403).json({ message: 'Usuário não encontrado' });
        }

        req.user = {
            id: user.id,
            roles: user.roles.map(role => role.name), // Mapeie as funções do usuário
        };

        next();
    } catch (error) {
        return res.status(403).json({ message: error.message });
    }
};
