import express from 'express';
import { ensuredAuthenticated } from '../middlewares/ensuredAuthenticated';

import multer from 'multer';
import path from 'path';
import EquipamentoController from '../controllers/EquipamentoController';

// Configuração do Multer para armazenar arquivos no disco
const storage = multer.memoryStorage();
const upload = multer({ storage: storage });

const router = express.Router();

router.get('/', ensuredAuthenticated(), EquipamentoController.listAll);
router.get('/tipo/equipamentos/filtro/:tipo_equipamento_id', ensuredAuthenticated(), EquipamentoController.list);
router.get('/tipo/:tipo_equipamento_id/:viatura_id', ensuredAuthenticated(), EquipamentoController.listEquipamentosUnidade);
router.get('/:id', ensuredAuthenticated(), EquipamentoController.show);
router.post('/', ensuredAuthenticated(), upload.single('imagem'), EquipamentoController.create);
router.put('/:id', ensuredAuthenticated(), upload.single('imagem'), EquipamentoController.update);
router.delete('/:id', ensuredAuthenticated(), EquipamentoController.delete);

router.post('/outro', ensuredAuthenticated(), upload.array('imagens', 10), EquipamentoController.create);
router.put('/outro/:id', ensuredAuthenticated(), upload.array('imagens', 10), EquipamentoController.update);


export default router;
