import express from 'express';
import { ensuredAuthenticated } from '../middlewares/ensuredAuthenticated';
import ViaturaController from '../controllers/ViaturaController';
import multer from 'multer';
import path from 'path';

// Configuração do Multer para armazenar arquivos no disco
const storage = multer.memoryStorage();
const upload = multer({ storage: storage });

const router = express.Router();

router.get('/', ensuredAuthenticated(), ViaturaController.list);
router.get('/:id', ensuredAuthenticated(), ViaturaController.show);
router.post('/', ensuredAuthenticated(), upload.array('imagens', 10), ViaturaController.create);
router.put('/:id', ensuredAuthenticated(), upload.array('imagens', 10), ViaturaController.update);
router.delete('/:id', ensuredAuthenticated(), ViaturaController.delete);
router.get('/codigo/:codigo', ensuredAuthenticated(), ViaturaController.buscarPorCodigo);
router.get('/unidade/:unidade_id', ensuredAuthenticated(), ViaturaController.listarViaturas);


export default router;
