import { Not } from "typeorm";
import { Permission } from "../entities/Permission";
import { PermissionRepository } from "../repositories";


type PermissionRequest = {
    id?: string;
    name: string;
    description: string;
};

export class CreatePermissionService {
    async execute({
        name,
        description,
    }: PermissionRequest): Promise<Permission | Error> {
        const repo = PermissionRepository();

        if (await repo.findOne({ name })) {
            return new Error("Permission already exists");
        }

        const permission = repo.create({ name, description });

        await repo.save(permission);

        return permission;
    }

    async update({ id, name, description }: PermissionRequest): Promise<Permission | Error> {
        const repo = PermissionRepository();

        const permmissionExistente = await repo.findOne({
            where: {
                id: id,
            },
        });

        if (!permmissionExistente) {
            return new Error("ID not exists");
        }

        if (await repo.findOne({
            where:
                { id: Not(id), name: name }
        })) {
            return new Error("Permission already exists");
        }

        const permission = repo.create({ name, description });

        await repo.update(id, permission);

        return permission;
    }
}