import { Not } from "typeorm";
import { Role } from "../entities/Role";
import { RoleRepository } from "../repositories";

type RoleRequest = {
    id?: string;
    name: string;
    description: string;
};

export class CreateRoleService {
    async execute({ name, description }: RoleRequest): Promise<Role | Error> {
        const repo = RoleRepository();

        if (await repo.findOne({ name })) {
            return new Error("Role already exists");
        }

        const role = repo.create({ name, description });

        await repo.save(role);

        return role;
    }

    async update({ id, name, description }: RoleRequest): Promise<Role | Error> {
        const repo = RoleRepository();

        const roleExistente = await repo.findOne({
            where: {
                id: id,
            },
        });

        if (!roleExistente) {
            return new Error("ID not exists");
        }

        if (await repo.findOne({
            where:
                { id: Not(id), name: name }
        })) {
            return new Error("Role already exists");
        }

        const role = repo.create({ name, description });

        await repo.update(id, role);

        return role;
    }
}