import { User } from "../entities/User";
import { hash } from "bcrypt";
import {
    PermissionRepository,
    RoleRepository,
    UserRepository,
} from "../repositories";

type UserACLRequest = {
    userId: string;
    roles: string[];
    permissions: string[];
    fullName?: string;
    email?: string;
    password?: string;
};

export class CreateUserAccessControlListService {
    async execute({
        userId,
        roles,
        permissions,
        fullName,
        email,
        password
    }: UserACLRequest): Promise<User | Error> {
        const repo = UserRepository();

        const user = await repo.findOne(userId);

        if (!user) {
            return new Error("User does not exists!");
        }

        let permissionsExists;
        if (permissions != null && permissions != undefined) {
            permissionsExists = await PermissionRepository().findByIds(
                permissions
            );
            user.permissions = permissionsExists;
        }


        const rolesExists = await RoleRepository().findByIds(roles);

        if (fullName != null && fullName != undefined) {
            user.fullName = fullName;
        }
        if (email != null && email != undefined) {
            user.email = email;
        }
        if (password != null && password != undefined) {
            const passwordHash = await hash(password, 8);
            user.password = passwordHash;
        }
        user.permissions = permissionsExists;
        user.roles = rolesExists;

        repo.save(user);

        return user;
    }
}