import { hash } from "bcrypt";
import { getRepository } from "typeorm";
import { User } from "../entities/User";
import { UserRepository } from "../repositories";

type UserRequest = {
    username: string;
    password: string;
    fullName: string;
    email: string;
    hasApiAccess: boolean;
};

export class CreateUserService {
    async execute({ password, username, fullName, email, hasApiAccess }: UserRequest): Promise<Error | User> {
        const existUser = await UserRepository().findOne({ email });

        if (existUser) {
            return new Error("User already exists");
        }

        const passwordHash = await hash(password, 8);

        const user = UserRepository().create({ username, password: passwordHash, fullName, email, hasApiAccess });

        await UserRepository().save(user);

        return user;
    }
}